@@ -1,12 +1,17 @@
 import asyncio
 import contextlib
 import gc
-import pytest
+import http.cookies
+import re
+import types
 from unittest import mock
 
+import pytest
+from multidict import CIMultiDict, MultiDict
+
 import aiohttp
+from aiohttp import web
 from aiohttp.client import ClientSession
-from aiohttp.multidict import MultiDict, CIMultiDict
 from aiohttp.connector import BaseConnector, TCPConnector
 
 
@@ -89,18 +94,20 @@ def test_init_headers_list_of_tuples_with_duplicates(create_session):
 def test_init_cookies_with_simple_dict(create_session):
     session = create_session(cookies={"c1": "cookie1",
                                       "c2": "cookie2"})
-    assert set(session.cookies) == {'c1', 'c2'}
-    assert session.cookies['c1'].value == 'cookie1'
-    assert session.cookies['c2'].value == 'cookie2'
+    cookies = session.cookie_jar.filter_cookies('')
+    assert set(cookies) == {'c1', 'c2'}
+    assert cookies['c1'].value == 'cookie1'
+    assert cookies['c2'].value == 'cookie2'
 
 
 def test_init_cookies_with_list_of_tuples(create_session):
     session = create_session(cookies=[("c1", "cookie1"),
                                       ("c2", "cookie2")])
 
-    assert set(session.cookies) == {'c1', 'c2'}
-    assert session.cookies['c1'].value == 'cookie1'
-    assert session.cookies['c2'].value == 'cookie2'
+    cookies = session.cookie_jar.filter_cookies('')
+    assert set(cookies) == {'c1', 'c2'}
+    assert cookies['c1'].value == 'cookie1'
+    assert cookies['c2'].value == 'cookie2'
 
 
 def test_merge_headers(create_session):
@@ -265,15 +272,10 @@ def test_connector_loop(loop):
         stack.enter_context(contextlib.closing(another_loop))
         connector = TCPConnector(loop=another_loop)
         stack.enter_context(contextlib.closing(connector))
-        with pytest.raises_regexp(
-                ValueError,
-                "loop argument must agree with connector"):
+        with pytest.raises(ValueError) as ctx:
             ClientSession(connector=connector, loop=loop)
-
-
-def test_cookies_are_readonly(session):
-    with pytest.raises(AttributeError):
-        session.cookies = 123
+        assert re.match("loop argument must agree with connector",
+                        str(ctx.value))
 
 
 def test_detach(session):
@@ -288,7 +290,7 @@ def test_detach(session):
         conn.close()
 
 
-@pytest.mark.run_loop
+@asyncio.coroutine
 def test_request_closed_session(session):
     session.close()
     with pytest.raises(RuntimeError):
@@ -312,12 +314,12 @@ def test_double_close(connector, create_session):
     assert connector.closed
 
 
-def test_del(connector, loop, warning):
+def test_del(connector, loop):
     # N.B. don't use session fixture, it stores extra reference internally
     session = ClientSession(connector=connector, loop=loop)
     loop.set_exception_handler(lambda loop, ctx: None)
 
-    with warning(ResourceWarning):
+    with pytest.warns(ResourceWarning):
         del session
         gc.collect()
 
@@ -337,7 +339,7 @@ def test_borrow_connector_loop(connector, create_session, loop):
         session.close()
 
 
-@pytest.mark.run_loop
+@asyncio.coroutine
 def test_reraise_os_error(create_session):
     err = OSError(1, "permission error")
     req = mock.Mock()
@@ -356,3 +358,65 @@ def test_reraise_os_error(create_session):
     e = ctx.value
     assert e.errno == err.errno
     assert e.strerror == err.strerror
+
+
+@asyncio.coroutine
+def test_request_ctx_manager_props(loop):
+    yield from asyncio.sleep(0, loop=loop)  # to make it a task
+    with aiohttp.ClientSession(loop=loop) as client:
+        ctx_mgr = client.get('http://example.com')
+
+        next(ctx_mgr)
+        assert isinstance(ctx_mgr.gi_frame, types.FrameType)
+        assert not ctx_mgr.gi_running
+        assert isinstance(ctx_mgr.gi_code, types.CodeType)
+
+
+@asyncio.coroutine
+def test_cookie_jar_usage(create_app_and_client):
+    req_url = None
+
+    jar = mock.Mock()
+    jar.filter_cookies.return_value = None
+
+    @asyncio.coroutine
+    def handler(request):
+        nonlocal req_url
+        req_url = "http://%s/" % request.host
+
+        resp = web.Response()
+        resp.set_cookie("response", "resp_value")
+        return resp
+
+    app, client = yield from create_app_and_client(
+        client_params={"cookies": {"request": "req_value"},
+                       "cookie_jar": jar}
+    )
+    app.router.add_route('GET', '/', handler)
+
+    # Updating the cookie jar with initial user defined cookies
+    jar.update_cookies.assert_called_with({"request": "req_value"})
+
+    jar.update_cookies.reset_mock()
+    yield from client.get("/")
+
+    # Filtering the cookie jar before sending the request,
+    # getting the request URL as only parameter
+    jar.filter_cookies.assert_called_with(req_url)
+
+    # Updating the cookie jar with the response cookies
+    assert jar.update_cookies.called
+    resp_cookies = jar.update_cookies.call_args[0][0]
+    assert isinstance(resp_cookies, http.cookies.SimpleCookie)
+    assert "response" in resp_cookies
+    assert resp_cookies["response"].value == "resp_value"
+
+
+def test_session_default_version(loop):
+    session = aiohttp.ClientSession(loop=loop)
+    assert session.version == aiohttp.HttpVersion11
+
+
+def test_session_loop(loop):
+    session = aiohttp.ClientSession(loop=loop)
+    assert session.loop is loop
